;;; navi2ch-message.el --- write message module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(defvar navi2ch-message-buffe-name "*navi2ch message*")
(defvar navi2ch-message-current-article-buffer nil)
(defvar navi2ch-message-current-article nil)
(defvar navi2ch-message-current-bord nil)
(defvar navi2ch-message-new-message-p nil)
(defvar navi2ch-message-window-configuration nil)
(defvar navi2ch-message-user-name
  (if (featurep 'xemacs)
      "$BL>L5$7$5$s!w#X#E#m#a#c#s(B"
    "$BL>L5$7$5$s!w#E#m#a#c#s(B")
  "$BL>A0(B")
(defvar navi2ch-message-remember-user-name t)
(defvar navi2ch-message-cite-prefix "> " "$B0zMQ$9$k$H$-$N@\F,<-(B")
(defvar navi2ch-message-mode-map nil)

(unless navi2ch-message-mode-map
  (setq navi2ch-message-mode-map (make-sparse-keymap))
  (define-key navi2ch-message-mode-map "\C-c\C-c" 'navi2ch-message-send-message)
  (define-key navi2ch-message-mode-map "\C-c\C-k" 'navi2ch-message-exit)
  (define-key navi2ch-message-mode-map "\C-c\C-y" 'navi2ch-message-cite-original)
  (define-key navi2ch-message-mode-map "\C-c\y" 'navi2ch-message-cite-original-from-number))

(defun navi2ch-message-write-message (bord article &optional new)
  (setq navi2ch-message-window-configuration
        (current-window-configuration))
  (delete-other-windows)
  (split-window-vertically)
  (other-window 1)
  (setq navi2ch-message-current-article article)
  (setq navi2ch-message-current-bord bord)
  (setq navi2ch-message-new-message-p new)
  (setq navi2ch-message-current-article-buffer
        (if new nil (current-buffer)))
  (switch-to-buffer (get-buffer-create navi2ch-message-buffe-name))
  (navi2ch-message-mode)
  (erase-buffer)
  (navi2ch-message-insert-header new))

(defun navi2ch-message-insert-header (new)
  (when new
    (insert "Subject: \n"))
  (insert "From: "
	  (or (cdr (assq 'name navi2ch-message-current-article))
              navi2ch-message-user-name) "\n"
          "Mail: \n"
          (make-string (1- (window-width)) ?-) "\n"))
  
(defun navi2ch-message-send-message ()
  (interactive)
  (save-excursion
    (let (subject from mail message)
      (goto-char (point-min))
      (when (re-search-forward "Subject: \\([^\n]*\\)\n" nil t)
        (setq subject (match-string 1)))
      (re-search-forward "From: \\([^\n]*\\)\n")
      (setq from (match-string 1))
      (when navi2ch-message-remember-user-name
        (setq navi2ch-message-user-name from))
      (when (not navi2ch-message-new-message-p)
        (navi2ch-message-set-name from))
      (re-search-forward "Mail: \\([^\n]*\\)\n")
      (setq mail (match-string 1))
      (forward-line 1)
      (setq message (buffer-substring (point) (point-max)))
      (when (navi2ch-net-send-message from mail message subject
                                      navi2ch-message-current-bord
                                      navi2ch-message-current-article)
        (save-excursion
          (if navi2ch-message-new-message-p
              (progn
                (set-buffer navi2ch-board-buffer-name)
                (navi2ch-board-sync))
            (set-buffer (navi2ch-article-current-buffer))
            (navi2ch-article-sync))))))
  (navi2ch-message-exit))

(defun navi2ch-message-set-name (name)
  (save-excursion
    (set-buffer navi2ch-message-current-article-buffer)
    (setq navi2ch-article-current-article
          (navi2ch-put-alist 'name name
                             navi2ch-article-current-article))))

(defun navi2ch-message-cite-original ()
  "$B0zMQ$9$k(B"
  (interactive)
  (navi2ch-message-cite-original-from-number
   (save-excursion
     (set-buffer (navi2ch-article-current-buffer))
     (navi2ch-article-get-current-number))))

(defun navi2ch-message-cite-original-from-number (num)
  "$BHV9f$rA*$s$G!"0zMQ$9$k!#(B"
  (interactive "ninput number: ")
  (insert-string ">>" num "\n")
  (let ((point (point)))
    (insert
     (save-excursion
       (set-buffer (navi2ch-article-current-buffer))
       (cdr (assq 'data (navi2ch-article-get-message num))))
     "\n")
    (string-rectangle point (point) navi2ch-message-cite-prefix)))
    
(defun navi2ch-message-exit ()
  (interactive)
  (kill-buffer (current-buffer))
  (set-window-configuration navi2ch-message-window-configuration))

(defun navi2ch-message-mode ()
  (interactive)
  (setq major-mode 'navi2ch-message-mode)
  (setq mode-name "Navi2ch Message")
  (use-local-map navi2ch-message-mode-map)
  (run-hooks 'navi2ch-message-mode-hook)
  (force-mode-line-update))

(provide 'navi2ch-message)

;;; navi2ch-message.el ends here