;;; navi2ch-util.el --- useful utilities for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'cl)
(require 'regexp-opt)

(defvar navi2ch-uudecode-program "uudecode")
(defvar navi2ch-uudecode-args nil)

(defvar navi2ch-replace-html-tag-regexp-alist
  '(("&gt;" . ">")
    ("&lt;" . "<")
    ("&quot;" . "\"")
    ("&amp" . "&")
    ("$B!w!.(B" . ","))
  "$BCV49$9$k(B html $B$N%?%0$N@55,I=8=$NO"A[%j%9%H(B")

(defvar navi2ch-replace-html-tag-regexp
  (regexp-opt (mapcar
               'car
               navi2ch-replace-html-tag-regexp-alist))
  "$BCV49$9$k(B html $B$N%?%0$N@55,I=8=(B
`navi2ch-replace-html-tag-regexp-alist' $B$+$i@8@.$5$l$k(B")

(defun navi2ch-bigint-int-to-list (i)
  (if (listp i)
      i
    (mapcar (lambda (x)
              (- x 48))
            (string-to-list (int-to-string i)))))

(defun navi2ch-bigint-multiply (a b)
  (setq a (reverse (navi2ch-bigint-int-to-list a))
        b (reverse (navi2ch-bigint-int-to-list b)))
  (let (list c)
    (dolist (y b)
      (let ((z 0))
        (setq list (cons
                    (append c (mapcar
                               (lambda (x)
                                 (let (w)
                                   (setq w (+ (* x y) z))
                                   (setq z (/ w 10))
                                   (mod w 10))) a)
                            (if (> z 0) (list z)))
                    list)))
      (setq c (cons 0 c)))
    (let (list2)
      (dolist (x list)
        (setq list2 (navi2ch-bigint-add list2 (reverse x))))
      list2)))

(defun navi2ch-bigint-add (a b)
  (setq a (reverse (navi2ch-bigint-int-to-list a))
        b (reverse (navi2ch-bigint-int-to-list b)))
  (let ((x 0) list)
    (while (or a b)
      (let (y)
        (setq y (+ (or (car a) 0) (or (car b) 0) x))
        (setq x (/ y 10))
        (setq list (cons (mod y 10) list))
        (setq a (cdr a)
              b (cdr b))))
    list))

(defun navi2ch-insert-file-contents (file &optional begin end)
  (let ((coding-system-for-read navi2ch-net-coding-system)
        (coding-system-for-write navi2ch-net-coding-system))
    (insert-file-contents file nil begin end)))

(defun navi2ch-expand-file-name (file)
  (expand-file-name file navi2ch-directory))
  
(defun navi2ch-replace-string (rep new str &optional all)
  (when str
    (if all
        (let (start)
          (while (setq start (string-match rep str start))
            (setq str (replace-match new nil nil str))))
      (when (string-match rep str)
        (setq str (replace-match new nil nil str))))
    str))
  
(defun navi2ch-uudecode-region (start end)
  (interactive "r")
  (let (dir)
    (save-window-excursion
      (delete-other-windows)
      (setq dir (read-file-name "directory name: ")))
    (unless (file-directory-p dir)
      (error "%s is not directory" dir))
    
    (let ((default-directory dir)
          (coding-system-for-read 'binary)
          (coding-system-for-write 'binary)
          rc)
      (setq rc (apply
                'call-process-region
                start end
                navi2ch-uudecode-program
                nil nil nil
                navi2ch-uudecode-args))
      (when (not (= rc 0))
        (error "uudecode error")))))

(defun navi2ch-read-number (prompt)
  "$B?t;z$r(B minibuffer $B$+$iFI$_9~$`(B"
  (catch 'loop
    (while t
      (let (elt)
        (setq elt (read-string prompt init history default))
        (cond ((string= elt "")
               (throw 'loop nil))
              ((string-match "^[ \t]*0+[ \t]*$" elt)
               (throw 'loop 0))
              ((not (eq (string-to-number elt) 0))
               (throw 'loop (string-to-int elt)))))
      (message "Please enter a number.")
      (sit-for 1))))

(defun navi2ch-replace-html-tag (str)
  (setq str (navi2ch-replace-string "<[^<]*>" "" str t))
  (let (start)
    (while (setq start (string-match
                        navi2ch-replace-html-tag-regexp
                        (or str "") start))
      (let ((new (cdr (assoc (match-string 0 str)
                             navi2ch-replace-html-tag-regexp-alist))))
        (setq str (replace-match new nil nil str)))))
  str)

(defun navi2ch-url-to-board (url)
  (when (string-match
         "http://\\([^/]+\\)/test/read\\.cgi.*bbs=\\([^&]+\\)"
         url)
    (list (cons 'name (match-string 2 url))
          (cons 'uri (concat "http://" (match-string 1 url)
                             "/" (match-string 2 url) "/"))
          (cons 'id (match-string 2 url)))))

(defun navi2ch-url-to-article (url)
  (when (string-match
         "http://[^/]+/test/read\\.cgi.*key=\\([^&]+\\)"
         url)
    (list (cons 'subject (match-string 1 url))
          (cons 'artid (match-string 1 url)))))

(defun navi2ch-2ch-url-p (url)
  (let (list)
    (setq list
          (mapcar
           (lambda (x)
             (let ((str (cdr (assq 'uri x))))
               (string-match "http://\\([^/]+\\)" str)
               (match-string 1 str)))
           navi2ch-list-board-name-list))
    (when (string-match "http://\\([^/]+\\)" url)
      (setq url (match-string 1 url))
      (member url list))))
                      
  
;; from apel
(defun navi2ch-put-alist (item value alist)
  "Modify ALIST to set VALUE to ITEM.
If there is a pair whose car is ITEM, replace its cdr by VALUE.
If there is not such pair, create new pair (ITEM . VALUE) and
return new alist whose car is the new pair and cdr is ALIST.
\[tomo's ELIS like function]"
  (let ((pair (assq item alist)))
    (if pair
	(progn
	  (setcdr pair value)
	  alist)
      (cons (cons item value) alist))))

(provide 'navi2ch-util)

;;; navi2ch-util.el ends here